#include "Common.h"
#include <exception>
using namespace std;

#ifndef __Process_h__
#define __Process_h__

typedef struct _ProcessEntry {
	PROCESSENTRY32 pe;
	BOOL bTriggered;
}ProcessEntry, *pProcessEntry;

// Limitiamo artificialmente la lunghezza del titolo a 512 caratteri
// lo facciamo perche' la continua allocazione e deallocazione della
// memoria sarebbe troppo onerosa per un cellulare.
typedef struct _WindowEntry {
	WCHAR wWindowTitle[MAX_TITLE_LEN];
	BOOL bTriggered;
}WindowEntry, *pWindowEntry;

class Process {
	private:
		static Process *Instance;
		static volatile LONG lLock;

		PROCESSENTRY32 pe;
		HANDLE hSnap, hProcessMutex, hWindowMutex;
		unsigned __int64 ulTime, ulWinTime;
		ProcessEntry procEntry;
		WindowEntry windowEntry;
		list<ProcessEntry> *pList;
		list<ProcessEntry>::iterator iter;
		static list<WindowEntry> *pWindowList; // Deve essere statico perche' viene usato in una callback
		list<WindowEntry>::iterator iterWindow;

	private:
		/**
		 * Refresha automaticamente la lista dei processi
		 */
		BOOL RefreshProcessList();

		/**
		* Refresha automaticamente la lista delle finestre
		*/
		BOOL RefreshWindowList();

		/**
		 * Funzione di callback per la EnumWindows()
		 */
		static BOOL CALLBACK EnumCallback(HWND hwnd, LPARAM lParam);

		/**
		 * La CompWildWildWest confronta stringhe con wildcard, torna 0 se sono diverse.
		 */
		INT CmpWildW(WCHAR *wild, WCHAR *string);

	public:
		/**
		* Process e' una classe singleton
		*/
		static Process* self();

		Process();
		~Process();
		UINT GetPid(WCHAR *pProcName);
		BOOL IsProcessRunning(WCHAR *pProcName);
		BOOL IsWindowPresent(WCHAR *pWindowName);
};

#endif